(***********************************************************************)
(*                                                                     *)
(*                               Grimage                               *)
(*                                                                     *)
(*                           Graphics image                            *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2014-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: colormapping.mli,v 1.7 2014/02/27 16:45:22 fclement Exp $ *)

(** {3 Colormappings} *)

(**
 A colormapping is a mapping from floating point numbers to colors.
 The colormapping contains a description of its domain and codomain and a
 function that maps numbers in the domain to colors in the codomain.
 The domain is a range of floating point numbers, described as the pair of
 the minimum and maximum values of the range.
 The codomain is described as a color scale which is an array of colors to
 which floating point numbers are mapped.
 The last field of a colormapping is the function itself, the [mapping] field.

 A color scaler interpolates between two colors in a specified number of steps.
 Interpolation method is a function of the color model.
*)

(** {6 Data type definitions} *)

type color_scale = Colors.t array;;
type from_float = float and to_float = float;;
type mapping = color_scale -> from_float -> to_float -> float -> Colors.t;;

type t =
  {
   color_scale : color_scale;
   from_float : from_float;
   to_float : to_float;
   mapping : mapping;
  }
;;
(** The type of colormappings.
 A [Colormapping.t] is made of a color scale (an array of colors)
 [color_scale], a range of floating point numbers [from_float..to_float]
 and a [mapping] function that maps any float value within the float range
 to the appropriate color in the [color_scale]. *)

type num_steps = int;;
type from_color = Colors.t and to_color = Colors.t;;

type color_scaler = from_color -> to_color -> num_steps -> int -> Colors.t;;
(** A [color_scaler] is a function [csr] such that [csr fc tc num_steps] maps
 any integer within the integer range [0..num_steps] to a color within the
 color range [fc..tc]. *)

(** {6 Scalers} *)

val scaler_rgb : color_scaler;;
(** [scaler_rgb fc tc w i] scales a color from color [fc] to color [tc], by
 [i] steps over [w] steps using the RGB representation of colors.
 [w] is supposed to be positive. *)

val scaler_hsb : color_scaler;;
(** Similar to [scaler_rgb] for HSB representation of colors. *)

val make_color_scale :
  color_scaler -> from_color -> to_color -> num_steps -> color_scale
;;
(** [make_color_scale csr fc tc w] returns the fresh array of colors
 of length [w], scaled from color [fc] (the first element of the array)
 to color [tc] (the last element of the array) according to the color
 scaler [csr]. *)

(** {6 Mappings} *)

val linear_mapping : mapping;;
(** [linear_mapping cs ff tf x] linearly interpolates between [ff] and [tf]
 to pick the color corresponding to [x] in the color scale [cs]. *)

(** {6 Colormappings} *)

val make : mapping -> color_scale -> from_float -> to_float -> t;;
(** [make m cs x y] makes a colormapping with mapping [m], array of colors
 [cs], and range of floats [x..y]. *)

val make_rgb :
  from_color -> to_color -> num_steps -> from_float -> to_float -> t
;;
(** [make_rgb fc tc w x y] returns the colormapping which linearly maps
 float range [x..y] to color range [fc..tc] in [w] steps, according to the RGB
 color representation.
 The corresponding color scale is the array of colors
 [make_color_scale scaler_rgb fc tc w].
 Values outside the range of the colormapping are automatically clipped into
 the range. *)

val make_hsb :
  from_color -> to_color -> num_steps -> from_float -> to_float -> t
;;
(** Similar to [make_rgb] for the HSB representation of colors. *)

val get_color : t -> float -> Colors.t;;
(** [get_color cm x] returns the color corresponding to [x] in colormapping
 [cm]. *)
